import enum

from fastapi_users_db_sqlalchemy import SQLAlchemyBaseUserTableUUID
from sqlalchemy import Column, String, DateTime, ForeignKey, BIGINT, UniqueConstraint, Integer
from sqlalchemy.dialects.postgresql import JSONB, ENUM
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql import expression
from sqlalchemy.sql.ddl import CreateColumn

from dataset_image_annotator.db import Base


class UTCNow(expression.FunctionElement):
    type = DateTime()


@compiles(UTCNow, 'postgresql')
def pg_utcnow(element, compiler, **kw):
    return "TIMEZONE('utc', CURRENT_TIMESTAMP)"


@compiles(CreateColumn, 'postgresql')
def use_identity(element, compiler, **kw):
    result = compiler.visit_create_column(element, **kw).replace('SERIAL', 'INT GENERATED BY DEFAULT AS IDENTITY')

    return result.replace('BIGSERIAL', 'BIGINT GENERATED BY DEFAULT AS IDENTITY')


class BaseDBModel:
    id = Column(Integer, primary_key=True)
    created_at = Column(DateTime, server_default=UTCNow())
    updated_at = Column(DateTime, onupdate=UTCNow())


class UserGroup(BaseDBModel, Base):
    __tablename__ = 'user_groups'

    name = Column(String, nullable=False)


class User(SQLAlchemyBaseUserTableUUID, Base):
    group_id = Column(Integer, ForeignKey('user_groups.id'), nullable=True)


class ImageSample(BaseDBModel, Base):
    __tablename__ = 'image_samples'

    checksum = Column(String, nullable=False)
    location = Column(String, nullable=False, unique=True)
    metadata = Column(JSONB)


class AnnotationKeyEnum(enum.Enum):
    type = 'type'
    make = 'make'
    model = 'model'
    body = 'body'
    color = 'color'


class ImageSampleAnnotation(BaseDBModel, Base):
    __tablename__ = 'image_sample_annotations'

    image_sample_id = Column(Integer, ForeignKey('image_samples.id'), nullable=False)
    key = Column(ENUM(AnnotationKeyEnum, name='enum_image_sample_annotation_key'), nullable=False)
    value = Column(String, nullable=False)
    votes = Column(Integer, nullable=False, default=0)
    user_id = Column(Integer, ForeignKey('image_samples.id'), nullable=False)

    __table_args__ = (
        UniqueConstraint('image_sample_id', 'key', 'user_id', name='uq_image_sample_annotation_item'),
    )



import uuid
from typing import Optional

from fastapi_users import schemas
from pydantic import BaseModel, UUID4


class UserRead(schemas.BaseUser[uuid.UUID]):
    group_id: Optional[int] = None


class UserCreate(schemas.BaseUserCreate):
    group_id: Optional[int] = None


class UserUpdate(schemas.BaseUserUpdate):
    group_id: Optional[int] = None


class UserGroupSearchParams(BaseModel):
    name: Optional[str]


class UserGroup(BaseModel):
    id: int
    name: str


class UserItem(BaseModel):
    id: UUID4
    email: str
    group_id: Optional[int]
    is_active: bool
    is_superuser: bool
    is_verified: bool



import argparse
import asyncio
import logging.config

from dataset_image_annotator.conf import settings


logging.config.dictConfig({
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': settings.logging_format,
        },
    },
    'handlers': {
        'default': {
            'level': settings.logging_level,
            'class': 'logging.StreamHandler',
            'stream': 'ext://sys.stdout',
            'formatter': 'standard',
        },
    },
    'loggers': {
        '': {
            'handlers': ['default'],
            'level': settings.logging_level,
            'propagate': True,
        }
    }
})

logger = logging.getLogger(__name__)


def get_parsed_args():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--job', type=str)

    args, args_other = parser.parse_known_args()

    return args


async def foo():
    logger.info('Foo')


async def bar():
    logger.info('Bar')


async def main():
    args = get_parsed_args()
    job_mapping = {
        'foo': foo,
        'bar': bar,
    }

    try:
        await job_mapping[args.job]()
    except KeyError:
        logger.error(f'Unknown job: "{args.job}"')


if __name__ == '__main__':
    asyncio.run(main())



import asyncio
import os
import sys
from logging.config import fileConfig

from alembic import context
from sqlalchemy import engine_from_config, pool
from sqlalchemy.ext.asyncio import AsyncEngine

from dataset_image_annotator.db.models import Base


sys.path.append(os.getcwd())

config = context.config

if config.attributes.get('configure_logger', True):
    fileConfig(config.config_file_name)

target_metadata = Base.metadata


def get_url():
    return os.getenv('DB_DSN', '')


def run_migrations_offline():
    """Run migrations in 'offline' mode.
    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.
    Calls to context.execute() here emit the given string to the
    script output.
    """
    url = get_url()
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        compare_type=True,
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection):
    context.configure(
        connection=connection, target_metadata=target_metadata, compare_type=True
    )

    with context.begin_transaction():
        context.run_migrations()


async def run_migrations_online():
    """Run migrations in 'online' mode.
    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    configuration = config.get_section(config.config_ini_section)
    configuration['sqlalchemy.url'] = get_url()
    connectable = AsyncEngine(
        engine_from_config(
            configuration, prefix='sqlalchemy.', poolclass=pool.NullPool, future=True,
        )
    )

    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migrations)


if context.is_offline_mode():
    run_migrations_offline()
else:
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        loop = None

    if loop and loop.is_running():
        tsk = loop.create_task(run_migrations_online())
        tsk.add_done_callback(lambda t: t.result())
    else:
        asyncio.run(run_migrations_online())



import logging

import sqlalchemy as sa
from fastapi_users.exceptions import UserAlreadyExists
from passlib import pwd

from dataset_image_annotator.api.users import create_user
from dataset_image_annotator.api.v1.schemas import UserCreate
from dataset_image_annotator.conf import settings
from dataset_image_annotator.db import database
from dataset_image_annotator.db.models import User

logger = logging.getLogger()


async def bootstrap_db():
    user_count = await database.fetch_val(sa.select([sa.func.count(User.id)]))

    if user_count == 0:
        logger.warning('No users in database')

        if settings.bootstrap_user_email:
            logger.info('Bootstrapping a user in database')
            password = pwd.genword(length=16, entropy=52)

            try:
                await create_user(
                    UserCreate(
                        email=settings.bootstrap_user_email,
                        password=password,
                        is_superuser=True,
                        is_active=True,
                        is_verified=True
                    )
                )
                logger.debug(f'Bootstrap user: {settings.bootstrap_user_email} / {password}')
            except UserAlreadyExists:
                logger.warning(f'User {settings.bootstrap_user_email} already exist')



import gettext
import logging
from pathlib import Path
from typing import Mapping, Optional, Sequence

import sqlalchemy as sa
from databases.backends.postgres import Record
from fastapi_pagination.bases import AbstractPage
from fastapi_pagination.ext.databases import paginate

from dataset_image_annotator.db.models import UserGroup, User

logger = logging.getLogger(__name__)
t = gettext.translation('base', Path(Path(__file__).parent, 'locale'), fallback=True, languages=['lv_LV'])
_ = t.gettext


async def get_users(database, search: Optional[Mapping[str, str]] = None,
                    order_by: Optional[str] = None) -> AbstractPage[Record]:
    query = sa.select([User])
    result = await paginate(database, query)

    return result


async def get_user(database, user_id: str) -> Record:
    query = sa.select([User]).where(User.id == user_id)

    return await database.fetch_row(query)


async def get_user_groups(database, search: Optional[Mapping[str, str]] = None,
                          order_by: Optional[str] = None) -> Sequence[Record]:
    query = sa.select([UserGroup]).order_by(UserGroup.name)
    result = await database.fetch_all(query)

    return result



import contextlib
import logging
import uuid
from typing import Optional

from fastapi import Request, Depends
from fastapi_users import BaseUserManager, UUIDIDMixin
from fastapi_users_db_sqlalchemy import SQLAlchemyUserDatabase

from dataset_image_annotator.api.v1.schemas import UserCreate, UserUpdate
from dataset_image_annotator.conf import settings
from dataset_image_annotator.db import database
from dataset_image_annotator.db.models import User

logger = logging.getLogger()


class UserManager(UUIDIDMixin, BaseUserManager[User, uuid.UUID]):
    reset_password_token_secret = settings.auth_secret
    verification_token_secret = settings.auth_secret

    async def on_after_register(self, user: User, request: Optional[Request] = None):
        logger.info(f'User {user.id} has registered.')

    async def on_after_forgot_password(self, user: User, token: str, request: Optional[Request] = None):
        logger.info(f'User {user.id} has forgot their password. Reset token: {token}')

    async def on_after_request_verify(self, user: User, token: str, request: Optional[Request] = None):
        logger.info(f'Verification requested for user {user.id}. Verification token: {token}')


@contextlib.asynccontextmanager
async def get_user_db_context():
    """Context manager usable in a general context"""
    yield SQLAlchemyUserDatabase(database, User.__table__)

    print("Close the db...")


async def get_user_db():
    """Dependency to use in a FastAPI context"""
    async with get_user_db_context() as user_db:
        yield user_db


@contextlib.asynccontextmanager
async def get_user_manager_context(user_db):
    """Context manager usable in a general context"""
    yield UserManager(user_db)


async def get_user_manager(user_db: SQLAlchemyUserDatabase = Depends(get_user_db)):
    """Dependency to use in a FastAPI context"""
    async with get_user_manager_context(user_db) as user_manager:
        yield user_manager


async def create_user(user: UserCreate):
    async with get_user_db_context() as user_db:
        async with get_user_manager_context(user_db) as user_manager:
            await user_manager.create(user)


async def update_user(user_id: str, user: UserUpdate):
    user.id = user_id

    async with get_user_db_context() as user_db:
        async with get_user_manager_context(user_db) as user_manager:
            r = await user_manager.update(user)

            logger.info(f'updated: {r}')

    return user



import logging
import uuid
from functools import wraps
from inspect import signature
from typing import Optional, Any, List

from fastapi import APIRouter, HTTPException, status, Depends
from fastapi.responses import ORJSONResponse
from fastapi_pagination import Page
from fastapi_users import FastAPIUsers
from fastapi_users.authentication import AuthenticationBackend, JWTStrategy, CookieTransport
from pydantic import Json

from dataset_image_annotator import core
from dataset_image_annotator.api import users
from dataset_image_annotator.api.users import get_user_manager
from dataset_image_annotator.api.v1.schemas import (
    UserCreate, UserUpdate, UserItem, UserGroup, UserRead
)
from dataset_image_annotator.conf import settings
from dataset_image_annotator.db import database
from dataset_image_annotator.db.models import User
from dataset_image_annotator.helpers import connect_to_db

logger = logging.getLogger(__name__)
router = APIRouter()
cookie_transport = CookieTransport(cookie_max_age=3600)


def get_jwt_strategy() -> JWTStrategy:
    return JWTStrategy(secret=settings.auth_secret, lifetime_seconds=3600)


auth_backend = AuthenticationBackend(name='cluserauth', transport=cookie_transport, get_strategy=get_jwt_strategy)
auth_backends = [auth_backend, ]
fastapi_users = FastAPIUsers[User, uuid.UUID](
    get_user_manager,
    auth_backends
)
get_current_user = fastapi_users.current_user(active=True)
auth_router = fastapi_users.get_auth_router(auth_backend, requires_verification=True)
users_router = fastapi_users.get_users_router(UserRead, UserUpdate, requires_verification=True)


@router.on_event('startup')
async def startup():
    await connect_to_db(database)


@router.on_event('shutdown')
async def shutdown():
    await database.disconnect()


def _handle_exceptions_helper(status_code, *args):
    if args:
        raise HTTPException(status_code=status_code, detail=args[0])
    else:
        raise HTTPException(status_code=status_code)


def handle_exceptions(func):
    signature(func)

    @wraps(func)
    async def wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except PermissionError as e:
            return _handle_exceptions_helper(status.HTTP_401_UNAUTHORIZED, *e.args)
        except LookupError as e:
            return _handle_exceptions_helper(status.HTTP_404_NOT_FOUND, *e.args)
        except ValueError as e:
            return _handle_exceptions_helper(status.HTTP_400_BAD_REQUEST, *e.args)

    return wrapper


@router.get('/users', response_model=Page[UserItem], response_class=ORJSONResponse, tags=['Admin'])
@handle_exceptions
async def get_users(search: Optional[Json] = None, order_by: Optional[str] = None,
                    user=Depends(get_current_user)):
    if user.is_superuser:
        return await core.get_users(database, search, order_by)

    raise HTTPException(status_code=403)


@router.post('/users', response_model=UserItem, response_class=ORJSONResponse, tags=['Admin'])
@handle_exceptions
async def create_user(new_user: UserCreate, user=Depends(get_current_user)):
    if user.is_superuser:
        created_user = await users.create_user(new_user)

        return created_user

    raise HTTPException(status_code=403)


@router.get('/user-groups', response_model=List[UserGroup], response_class=ORJSONResponse, tags=['Admin'])
@handle_exceptions
async def get_user_groups(search: Optional[Json[Any]] = None, order_by: Optional[str] = None,
                          user=Depends(get_current_user)):
    if user.is_active:
        return await core.get_user_groups(database, search, order_by)

    raise HTTPException(status_code=403)



import logging

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi_pagination import add_pagination

from dataset_image_annotator.api.v1.endpoints import router, auth_router, users_router

origins = [
    'http://localhost',
    'http://localhost:3000',
    'http://localhost:8080',
    'http://localhost:5000',
]

logger = logging.getLogger(__name__)
app = FastAPI(docs_url='/api/docs', openapi_url='/api/v1/openapi.json')
app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=['*'],
    allow_headers=['*'],
)
app.include_router(router, prefix='/api/v1')
app.include_router(
    auth_router,
    prefix='/api/v1/auth',
    tags=['auth'],
)
app.include_router(
    users_router,
    prefix='/api/v1/users',
    tags=['users'],
)
add_pagination(app)



import argparse
import json
import sys
from collections import defaultdict
from pathlib import Path
from typing import Sequence, Union, Optional, Mapping

import rawpy
from PySide6.QtCore import QFile, QIODevice, QDir, QFileInfo, QModelIndex, Qt, QStringListModel
from PySide6.QtGui import QPixmap, QIcon
from PySide6.QtUiTools import QUiLoader
from PySide6.QtWidgets import (
    QApplication, QGraphicsScene, QFileDialog, QFileSystemModel, QListView, QFileIconProvider, QCompleter
)


def get_parsed_args():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--data-root', type=str)
    # parser.add_argument('--datasets', metavar='DS', type=str, nargs='+')

    args, args_other = parser.parse_known_args()

    return args


def list_dir(path: Path, mask: str) -> Sequence[Path]:
    if path.is_dir():
        files = sorted(path.glob(mask))
    else:
        raise FileNotFoundError('Path must be directory')

    return files


def list_dir_images(path: Path) -> Sequence[Path]:
    return list_dir(path, '*.ARW')


def list_dir_metadata(path: Path) -> Optional[Sequence[Path]]:
    metadata_dir_path = Path(path, '.metadata')

    if metadata_dir_path.exists():
        return list_dir(metadata_dir_path, '*.json')

    return None


def load_metadata(data_root_path: Path):
    metadata = defaultdict(dict)
    metadata_files = list_dir_metadata(data_root_path)

    if metadata_files:
        for metadata_file in metadata_files:
            try:
                with open(metadata_file, 'r') as f:
                    metadata[metadata_file.name.lower()] = json.load(f)
            except FileNotFoundError:
                pass

    return metadata


def save_metadata(data_root_path: Path, file_name: str, metadata: Mapping):
    metadata_dir_path = Path(data_root_path, '.metadata')
    metadata_file_path = Path(metadata_dir_path, f'{file_name.lower()}.json')

    metadata_dir_path.mkdir(exist_ok=True)

    with open(metadata_file_path, 'w') as f:
        json.dump(metadata, f)


def get_raw_thumbnail(path: Union[str, Path]):
    with rawpy.imread(str(path)) as raw:
        try:
            thumb = raw.extract_thumb()
        except rawpy.LibRawNoThumbnailError:
            print('no thumbnail found')
        except rawpy.LibRawUnsupportedThumbnailError:
            print('unsupported thumbnail')
        else:
            return thumb


class RawIconProvider(QFileIconProvider):
    def icon(self, file_info: QFileInfo) -> QIcon:
        if isinstance(file_info, QFileInfo):
            file_path = file_info.absoluteFilePath()
            file_name = file_info.fileName().lower()

            if file_name.endswith('.arw'):
                thumbnail_dir_path = Path(f'{Path(file_path).parent}/.thumbs')

                thumbnail_dir_path.mkdir(exist_ok=True)
                thumbnail_path = Path(f'{thumbnail_dir_path}/{file_name}.png')
                thumb_pixmap = QPixmap()

                if thumbnail_path.exists():
                    thumb_pixmap.load(str(thumbnail_path))
                else:
                    thumbnail = get_raw_thumbnail(file_path)
                    thumb_pixmap.loadFromData(thumbnail.data)
                    thumb_pixmap = thumb_pixmap.scaledToWidth(80)

                    f = QFile(thumbnail_path)
                    f.open(QIODevice.OpenModeFlag.WriteOnly)
                    thumb_pixmap.save(f, 'PNG')

                return QIcon(thumb_pixmap)

            return super().icon(self.IconType.File)


class MainWindow:
    def __init__(self, data_root_path: Path):
        self.data_root_path = None
        self.metadata = defaultdict(dict)
        self.selected_file_name: Optional[str] = None
        self.types = set()
        self.makes = set()
        self.models = set()
        self.bodies = set()
        self.colors = set()

        ui_file_name = 'main_window.ui'
        ui_file = QFile(Path(__file__).resolve().parent / ui_file_name)

        if not ui_file.open(QIODevice.OpenModeFlag.ReadOnly):
            print(f'Cannot open {ui_file_name}: {ui_file.errorString()}')
            sys.exit(-1)

        loader = QUiLoader()
        self.window = loader.load(ui_file)
        ui_file.close()

        self.type_completer = QCompleter()
        self.type_completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        self.type_completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.type_completer.setWidget(self.window.type_combo_box)
        self.window.type_combo_box.editTextChanged.connect(self.on_type_changed)

        self.make_completer = QCompleter()
        self.make_completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        self.make_completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.make_completer.setWidget(self.window.make_combo_box)
        self.window.make_combo_box.editTextChanged.connect(self.on_make_changed)

        self.model_completer = QCompleter()
        self.model_completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        self.model_completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.model_completer.setWidget(self.window.model_combo_box)
        self.window.model_combo_box.editTextChanged.connect(self.on_model_changed)

        self.body_completer = QCompleter()
        self.body_completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        self.body_completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.body_completer.setWidget(self.window.body_combo_box)
        self.window.body_combo_box.editTextChanged.connect(self.on_body_changed)

        self.color_completer = QCompleter()
        self.color_completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        self.color_completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.color_completer.setWidget(self.window.color_combo_box)
        self.window.color_combo_box.editTextChanged.connect(self.on_color_changed)

        self.window.path_browser_button.clicked.connect(self.browse_directory)
        self.window.path_edit.textChanged.connect(self.on_data_root_path_changed)

        if data_root_path:
            self.window.path_edit.setText(str(data_root_path))

        if not self.window:
            print(loader.errorString())
            sys.exit(-1)

        self.window.thumbnail_list_view.clicked.connect(self.on_file_selected)
        self.window.showMaximized()

    def browse_directory(self):
        path = QFileDialog.getExistingDirectory(self.window, 'Select dataset directory', str(self.data_root_path),
                                                options=QFileDialog.Option.ShowDirsOnly)
        self.window.path_edit.setText(path)


    def on_metadata_property_changed(self, key: str, value: str):
        current_metadata = self.metadata[self.selected_file_name.lower()]
        current_metadata[key] = value.lower()
        save_metadata(self.data_root_path, self.selected_file_name, current_metadata)


    def on_type_changed(self, value: str):
        self.on_metadata_property_changed('type', value)

    def on_make_changed(self, value: str):
        self.on_metadata_property_changed('make', value)


    def on_model_changed(self, value: str):
        self.on_metadata_property_changed('model', value)


    def on_body_changed(self, value: str):
        self.on_metadata_property_changed('body', value)


    def on_color_changed(self, value: str):
        self.on_metadata_property_changed('color', value)


    def on_file_selected(self, index: QModelIndex):
        self.selected_file_name = index.data()
        file_path = Path(self.data_root_path, self.selected_file_name)

        thumb = get_raw_thumbnail(file_path)
        thumb_pixmap = QPixmap()
        thumb_pixmap.loadFromData(thumb.data)

        scene = QGraphicsScene()
        scene.addPixmap(thumb_pixmap)
        self.window.photo_view.setScene(scene)

        self.window.type_combo_box.setEnabled(True)
        self.window.make_combo_box.setEnabled(True)
        self.window.model_combo_box.setEnabled(True)
        self.window.body_combo_box.setEnabled(True)
        self.window.color_combo_box.setEnabled(True)



    def on_data_root_path_changed(self):
        self.data_root_path = Path(self.window.path_edit.text())
        self.load_images(self.data_root_path)
        self.metadata = load_metadata(self.data_root_path)

        self.types = set()
        self.makes = set()
        self.models = set()
        self.bodies = set()
        self.colors = set()

        for item in self.metadata.values():
            if _type := item.get('type'):
                self.types.add(_type)

            if make := item.get('make'):
                self.makes.add(make)

            if model := item.get('model'):
                self.models.add(model)

            if body := item.get('body'):
                self.bodies.add(body)

            if color := item.get('color'):
                self.colors.add(color)

        self.window.type_combo_box.clear()
        self.window.type_combo_box.addItems(sorted(self.types))
        self.type_completer.setModel(QStringListModel(sorted(self.types)))
        self.window.type_combo_box.setCompleter(self.type_completer)

        self.window.make_combo_box.clear()
        self.window.make_combo_box.addItems(sorted(self.makes))
        self.make_completer.setModel(QStringListModel(sorted(self.makes)))
        self.window.make_combo_box.setCompleter(self.make_completer)

        self.window.model_combo_box.clear()
        self.window.model_combo_box.addItems(sorted(self.models))
        self.model_completer.setModel(QStringListModel(sorted(self.models)))
        self.window.model_combo_box.setCompleter(self.model_completer)

        self.window.body_combo_box.clear()
        self.window.body_combo_box.addItems(sorted(self.bodies))
        self.body_completer.setModel(QStringListModel(sorted(self.bodies)))
        self.window.body_combo_box.setCompleter(self.body_completer)

        self.window.color_combo_box.clear()
        self.window.color_combo_box.addItems(sorted(self.colors))
        self.color_completer.setModel(QStringListModel(sorted(self.colors)))
        self.window.color_combo_box.setCompleter(self.color_completer)

    def load_images(self, data_root_path: Path):
        image_file_paths = list_dir_images(data_root_path)

        if image_file_paths:
            model = QFileSystemModel()
            model.setFilter(QDir.Filter.Files)
            model.setNameFilters(('*.arw',))
            model.setNameFilterDisables(False)
            model.setRootPath(QDir.rootPath())
            model.setIconProvider(RawIconProvider())
            self.window.thumbnail_list_view.setViewMode(QListView.ViewMode.IconMode)
            self.window.thumbnail_list_view.setLayoutMode(QListView.LayoutMode.Batched)
            self.window.thumbnail_list_view.setBatchSize(20)
            self.window.thumbnail_list_view.setModel(model)
            self.window.thumbnail_list_view.setRootIndex(model.index(str(data_root_path)))


def main():
    args = get_parsed_args()
    data_root_path = Path(args.data_root).expanduser()

    app = QApplication(sys.argv)
    mainwindow = MainWindow(data_root_path)

    sys.exit(app.exec())


main()



import asyncio
import datetime
import logging
import os
import pathlib

from alembic import command
from alembic.config import Config
from asyncpg import CannotConnectNowError

from dataset_image_annotator.conf import settings

logger = logging.getLogger(__name__)


def date_from_string(string: str, fmt: str = '%d.%m.%Y') -> datetime.date:
    try:
        return datetime.datetime.strptime(string, fmt).date()
    except ValueError:
        return datetime.date.fromisoformat(string)


def datetime_from_string(string: str) -> datetime.datetime:
    try:
        return datetime.datetime.strptime(string, '%d.%m.%Y %H:%M:%S')
    except ValueError:
        return datetime.datetime.fromisoformat(string)


def date_range(start_date, end_date):
    for n in range(int((end_date - start_date).days + 1)):
        yield start_date + datetime.timedelta(days=n)


def run_db_migrations(config, dsn: str, script_location: str) -> None:
    logger.info(f'Running DB migrations in {script_location}')
    original_wd = os.getcwd()
    os.chdir(script_location)
    alembic_cfg = Config(config)
    alembic_cfg.attributes['configure_logger'] = False
    alembic_cfg.set_main_option('sqlalchemy.url', dsn)
    command.upgrade(alembic_cfg, 'head')
    os.chdir(original_wd)


async def connect_to_db(database):
    logger.info('Waiting for services')
    logger.debug(f'DB_DSN: {settings.db_dsn}')
    timeout = 0.001
    total_timeout = 0

    for i in range(15):
        try:
            await database.connect()
        except (ConnectionRefusedError, CannotConnectNowError):
            timeout *= 2
            await asyncio.sleep(timeout)
            total_timeout += timeout
        else:
            break
    else:
        msg = f'Unable to connect database for {int(total_timeout)}s'
        logger.error(msg)
        raise ConnectionRefusedError(msg)

    try:
        if settings.alembic_auto_upgrade and settings.alembic_config:
            script_location = str(pathlib.Path(__file__).parent.absolute())
            run_db_migrations(settings.alembic_config, settings.db_dsn, f'{script_location}/db')
        else:
            logger.info('Automatic DB migration is disabled')
    except Exception as e:
        logger.error(f'Automatic DB migration failed: {e}')


def tries(times):
    def func_wrapper(f):
        async def wrapper(*args, **kwargs):
            for time in range(times if times > 0 else 1):
                # noinspection PyBroadException
                try:
                    return await f(*args, **kwargs)
                except Exception as exc:
                    if time >= times:
                        raise exc

        return wrapper

    return func_wrapper



import sys

from pkg_resources import VersionConflict, require
from setuptools import setup

try:
    require('setuptools>=60.5')
except VersionConflict:
    print('Error: version of setuptools is too old (<60.5)!')
    sys.exit(1)

if __name__ == '__main__':
    setup(use_pyscaffold=True)



from typing import Optional

from pydantic import BaseSettings


class Settings(BaseSettings):
    logging_level: str = 'INFO'
    logging_format: str = '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
    db_dsn: Optional[str] = None
    service_addr: str = '127.0.0.1'
    service_port: int = 8080
    alembic_auto_upgrade: bool = False
    alembic_config: str = 'alembic.ini'
    bootstrap_user_email: Optional[str] = None
    auth_secret: str = 'TODO-REPLACE'


settings = Settings()



import uvicorn

from dataset_image_annotator.api.http import app
from dataset_image_annotator.conf import settings


if __name__ == '__main__':
    uvicorn.run(app, host=settings.service_addr, port=settings.service_port)



import databases
from sqlalchemy import MetaData
from sqlalchemy.ext.declarative import declarative_base

from dataset_image_annotator.conf import settings

metadata = MetaData()
Base = declarative_base(metadata=metadata)
database = databases.Database(settings.db_dsn)


async def is_healthy(pg) -> bool:
    return await pg.fetchval('SELECT 1 FROM alembic_version;') == 1
